import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Representa una mesa donde se apilan fichas de Mahjong
 */
public class Mesa extends World
{
    /** Los 4 tipos de palos de una baraja de naipes francesa. Utilizados para crear la baraja **/
    private String palos[] = {"bamboo", "man", "pin", "special"};
    
    /** Las fichas del juego, 4 por cada tipo **/
    private Ficha fichas[];
    
    /** La cantidad de replicas de una misma ficha. Debe ser un numero par mayor a 0 **/
    private static final int replicasFicha = 4;
    
    /** La cantidad de fichas que tiene cada palo, determinado por el numero de imagenes **/
    private static final int fichasPorPalo = 9;
    
    /**
     * Construye la mesa de juego, desordenada las fichas y los marcadores
     */
    public Mesa()
    {
        super(800, 600, 1);
        
        crearFondo();
        crearFichas();
        desordenar();
        mostrarFichas();
    }
    
    /**
     * El fondo del escenario es una mesa, simulando que tiene una alfombra verde
     */
    private void crearFondo()
    {
        GreenfootImage fondo = getBackground();
        fondo.setColor( new java.awt.Color(0, 100, 0) );
        fondo.fillRect(0, 0, getWidth(), getHeight() );
    }
    
    /**
     * Crea cada una de las 52 cartas que conforman la baraja. Estas apareceran en orden en el
     * arreglo, primero por palo y luego por valor. Esto solo ocurre en la memoria de la
     * computadora y no tiene efecto visual en la pantalla
     */
    private void crearFichas()
    {
        // Cada palo tiene 9 figuras y se crean 4 fichas de cada una
        fichas = new Ficha[ palos.length * fichasPorPalo * replicasFicha ];
        int indice = 0;
        for ( int palo = 0; palo < palos.length; ++palo )
            for ( int i = 1; i <= fichasPorPalo; ++i )
                for ( int j = 0; j < replicasFicha; ++j )
                    fichas[indice++] = new Ficha(palos[palo] + i);
    }
    
    /**
     * Desordena las cartas en la baraja. Esto solo ocurre en la memoria de la
     * computadora y no tiene efecto visual en la pantalla
     */
    public void desordenar()
    {
        for ( int i = 0; i < fichas.length; ++i )
        {
            int otra = Greenfoot.getRandomNumber(fichas.length);
            if ( i != otra )
            {
                // intercambiar la carta en la posicion i con la carta en la posicion otra
                Ficha temporal = fichas[i];
                fichas[i] = fichas[otra];
                fichas[otra] = temporal;
            }
        }
    }
    
    /**
     * Muestra cada carta de la baraja en la mesa, de tal forma que todas se distribuyan en un
     * mosaico o matriz
     */
    private void mostrarFichas()
    {
        int indice = 0, x = 0, y = 0;
        for ( int palo = 0; palo < palos.length; ++palo )
            for ( int i = 1; i <= fichasPorPalo; ++i )
                for ( int j = 0; j < replicasFicha; ++j )
                {
                    Ficha ficha = fichas[indice++];
                    ficha.asignarNivelZ(palo);
                    boolean nueva = true;
                    do
                    {
                        x = 32 + Greenfoot.getRandomNumber(getWidth() - 64);
                        y = 48 + 32 + Greenfoot.getRandomNumber(getHeight() - 64 - 48);
                        if ( nueva ) addObject(ficha, x, y); else ficha.setLocation(x, y);
                        nueva = false;
                    } while ( ! ficha.estaLibre() );
                }
    }
}
